/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx;

import java.util.List;
import java.util.Map;

import org.ow2.petals.admin.api.RegistryAdministration;
import org.ow2.petals.admin.api.exception.RegistryAdministrationException;
import org.ow2.petals.admin.api.exception.RegistryRegexpPatternException;
import org.ow2.petals.admin.jmx.registry.JMXRegistryView;
import org.ow2.petals.admin.registry.RegistryView;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointRegistryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointRegistryServiceErrorException;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class JMXRegistryAdministration implements RegistryAdministration {

    protected JMXRegistryAdministration() {
    }

    @Override
    public RegistryView getRegistryContent(final String containerNameRegex, final String endpointNameRegex,
            final String serviceNameRegex, final String interfaceNameRegex) throws RegistryRegexpPatternException,
            RegistryAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            List<Map<String, Object>> endpointMaps = client.getEndpointRegistryClient()
                    .getAllEndpoints();
            return new JMXRegistryView(endpointMaps, containerNameRegex, endpointNameRegex, serviceNameRegex,
                    interfaceNameRegex);
        } catch (ConnectionErrorException cee) {
            throw new RegistryAdministrationException(cee);
        } catch (NumberFormatException nfe) {
            throw new RegistryAdministrationException(nfe);
        } catch (final EndpointRegistryServiceErrorException ersee) {
            throw new RegistryAdministrationException(ersee);
        } catch (final EndpointRegistryServiceDoesNotExistException ersdnee) {
            throw new RegistryAdministrationException(ersdnee);
        } catch (final RegistryException e) {
            throw new RegistryAdministrationException(e);
        }
    }

}
