/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx.artifact;

import java.net.URL;

import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.artifact.lifecycle.SharedLibraryLifecycle;
import org.ow2.petals.admin.api.conf.PetalsAdminConfiguration;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactStoppedException;
import org.ow2.petals.admin.api.exception.ArtifactUrlRewriterException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;

/**
 * This class represents a remote JBI shared library.
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class JMXSharedLibraryLifecycle extends AbstractLifecycle implements SharedLibraryLifecycle {

    private final SharedLibrary sharedLibrary;

    protected JMXSharedLibraryLifecycle(final SharedLibrary sl, final PetalsAdminConfiguration configuration) {
        super(configuration);
        this.sharedLibrary = sl;
    }

    public SharedLibrary getSharedLibrary() {
        return sharedLibrary;
    }

    @Override
    public void deploy(URL artifactUrl) throws ArtifactDeployedException,
            ArtifactAdministrationException {
        try {
            JMXClient client = JMXClientConnection.getJMXClient();
            client.getInstallationServiceClient().installSharedLibrary(
                    this.getConfiguration().getArtifactUrlRewriter().rewrite(artifactUrl));
        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final InstallationServiceErrorException isee) {
            throw new ArtifactAdministrationException(isee);
        } catch (final InstallationServiceDoesNotExistException isdnee) {
            throw new ArtifactAdministrationException(isdnee);
        } catch (final ArtifactUrlRewriterException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    @Override
    public void undeploy() throws ArtifactStartedException, ArtifactNotFoundException, ArtifactAdministrationException {

        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String slName = this.getSharedLibrary().getName();
            final String slVersion = this.getSharedLibrary().getVersion();
            final boolean isSuccessful = client.getInstallationServiceClient()
                    .uninstallSharedLibrary(slName, slVersion);
            if (!isSuccessful) {
                final String errorMessage = String.format("Failed to uninstall the SL '%s', '%s'", slName, slVersion);
                throw new ArtifactAdministrationException(errorMessage.toString());
            }
        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final InstallationServiceErrorException isee) {
            throw new ArtifactAdministrationException(isee);
        } catch (final InstallationServiceDoesNotExistException isdnee) {
            throw new ArtifactAdministrationException(isdnee);
        }
    }

    @Override
    public void updateState() throws ArtifactAdministrationException {
    }

    @Override
    public void start() throws ArtifactStartedException, ArtifactNotDeployedException,
            ArtifactNotFoundException, ArtifactAdministrationException {
    }

    @Override
    public void stop() throws ArtifactStoppedException, ArtifactNotDeployedException,
            ArtifactNotFoundException, ArtifactAdministrationException {
    }
}
